@@ -19,12 +19,15 @@
 # IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-
+from __future__ import absolute_import
 import os
 import sys
 import collections
 import importlib
-import cPickle
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 from hl7apy.exceptions import UnsupportedVersion, InvalidEncodingChars, UnknownValidationLevel
 from hl7apy.consts import DEFAULT_ENCODING_CHARS, DEFAULT_VERSION, VALIDATION_LEVEL
@@ -54,7 +57,7 @@ def check_encoding_chars(encoding_chars):
     if missing:
         raise InvalidEncodingChars('Missing required encoding chars')
 
-    values = [ v for k, v in encoding_chars.items() if k in required ]
+    values = [v for k, v in encoding_chars.items() if k in required]
     if len(values) > len(set(values)):
         raise InvalidEncodingChars('Found duplicate encoding chars')
 
@@ -70,6 +73,7 @@ def check_validation_level(validation_level):
     if validation_level not in (VALIDATION_LEVEL.QUIET, VALIDATION_LEVEL.STRICT, VALIDATION_LEVEL.TOLERANT):
         raise UnknownValidationLevel
 
+
 def check_version(version):
     """
     Validate the given version number
@@ -89,7 +93,7 @@ def get_default_encoding_chars():
     :rtype: ``dict``
     :returns: the encoding chars (see :func:`hl7apy.set_default_encoding_chars`)
 
-    >>> print get_default_encoding_chars()['FIELD']
+    >>> print(get_default_encoding_chars()['FIELD'])
     |
     """
     return _DEFAULT_ENCODING_CHARS
@@ -102,11 +106,12 @@ def get_default_version():
     :rtype: ``str``
     :returns: the default version
 
-    >>> print get_default_version()
+    >>> print(get_default_version())
     2.5
     """
     return _DEFAULT_VERSION
 
+
 def get_default_validation_level():
     """
     Get the default validation level
@@ -114,7 +119,7 @@ def get_default_validation_level():
     :rtype: ``str``
     :returns: the default validation level
 
-    >>> print get_default_validation_level()
+    >>> print(get_default_validation_level())
     2
     """
     return _DEFAULT_VALIDATION_LEVEL
@@ -128,12 +133,12 @@ def set_default_validation_level(validation_level):
     :param validation_level: validation level (see :class:`hl7apy.consts.VALIDATION_LEVEL`)
     :raises: :exc:`hl7apy.exceptions.UnknownValidationLevel` if the given validation level is unsupported
 
-    >>> set_default_validation_level(3)
+    >>> set_default_validation_level(3)  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     UnknownValidationLevel
     >>> set_default_validation_level(VALIDATION_LEVEL.TOLERANT)
-    >>> print get_default_validation_level()
+    >>> print(get_default_validation_level())
     2
     """
     check_validation_level(validation_level)
@@ -151,12 +156,12 @@ def set_default_version(version):
     :param version: the new default version (e.g. ``2.6``)
     :raises: :class:`hl7apy.exceptions.UnsupportedVersion` if the given version is unsupported
 
-    >>> set_default_version('22')
+    >>> set_default_version('22')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     UnsupportedVersion: The version 22 is not supported
     >>> set_default_version('2.3')
-    >>> print get_default_version()
+    >>> print(get_default_version())
     2.3
     """
     check_version(version)
@@ -194,17 +199,18 @@ def set_default_encoding_chars(encoding_chars):
     |ESCAPE        |``\\``            |
     +--------------+-----------------+
 
-    >>> set_default_encoding_chars({'FIELD': '!'})
+    >>> set_default_encoding_chars({'FIELD': '!'})  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InvalidEncodingChars: Missing required encoding chars
-    >>> set_default_encoding_chars({'FIELD': '!', 'COMPONENT': 'C', 'SUBCOMPONENT': 'S', 'REPETITION': 'R', 'ESCAPE': '\\\\'})
-    >>> print get_default_encoding_chars()['FIELD']
+    >>> set_default_encoding_chars({'FIELD': '!', 'COMPONENT': 'C', 'SUBCOMPONENT': 'S', \
+                                    'REPETITION': 'R', 'ESCAPE': '\\\\'})
+    >>> print(get_default_encoding_chars()['FIELD'])
     !
     """
     check_encoding_chars(encoding_chars)
 
-    encoding_chars.update({'GROUP' : '\r', 'SEGMENT' : '\r'})
+    encoding_chars.update({'GROUP': '\r', 'SEGMENT': '\r'})
 
     global _DEFAULT_ENCODING_CHARS
 
@@ -256,15 +262,15 @@ def load_reference(name, element_type, version):
     |              |('sequence', (<child>, (<min>, <max>), ...))|
     +--------------+--------------------------------------------+
 
-    >>> load_reference('UNKNOWN', 'Segment', '2.5')
+    >>> load_reference('UNKNOWN', 'Segment', '2.5')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     ChildNotFound: No child named UNKNOWN
     >>> r = load_reference('ADT_A01', 'Message', '2.5')
-    >>> print r[0]
+    >>> print(r[0])
     sequence
     >>> r = load_reference('MSH_3', 'Field', '2.5')
-    >>> print r[0]
+    >>> print(r[0])
     leaf
     """
     lib = load_library(version)
@@ -287,16 +293,16 @@ def find_reference(name, element_types, version):
     :raise: :class:`hl7apy.exceptions.ChildNotFound` if the element has not been found
 
     >>> from hl7apy.core import Message, Segment
-    >>> find_reference('UNKNOWN', (Segment, ), '2.5')
+    >>> find_reference('UNKNOWN', (Segment, ), '2.5')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     ChildNotFound: No child named UNKNOWN
-    >>> find_reference('ADT_A01', (Segment,),  '2.5')
+    >>> find_reference('ADT_A01', (Segment,),  '2.5')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     ChildNotFound: No child named ADT_A01
     >>> r = find_reference('ADT_A01', (Message,),  '2.5')
-    >>> print r['name'], r['cls']
+    >>> print('%s %s' % (r['name'], r['cls']))
     ADT_A01 <class 'hl7apy.core.Message'>
     """
     lib = load_library(version)
@@ -305,8 +311,8 @@ def find_reference(name, element_types, version):
 
 
 def load_message_profile(path):
-    with open(path) as f:
-        mp = cPickle.load(f)
+    with open(path, 'rb') as f:
+        mp = pickle.load(f)
 
     return mp
 
