@@ -32,11 +32,12 @@
 
 """
 
+from __future__ import absolute_import
 import re
 import numbers
 from datetime import datetime
-
 from decimal import Decimal
+from functools import cmp_to_key
 
 from hl7apy import get_default_encoding_chars, get_default_validation_level
 from hl7apy.exceptions import MaxLengthReached, InvalidHighlightRange, InvalidDateFormat, \
@@ -146,7 +147,7 @@ class TextualDataType(BaseDataType):
                 else:
                     raise InvalidHighlightRange(x, y)
 
-            self.highlights = sorted(self.highlights, cmp=_sort_highlights)
+            self.highlights = sorted(self.highlights, key=cmp_to_key(_sort_highlights))
             words = list(value)
             offset = 0
             for hl in self.highlights:
@@ -203,8 +204,8 @@ class DateTimeDataType(BaseDataType):
     :type value datetime: datetime.datetime
     :param value: a `datetime` date object. Default is ``None``
 
-    :type format: str
-    :param format: the format that will be used converting the object to string.
+    :type out_format: str
+    :param out_format: the format that will be used converting the object to string.
         It must be an item of the :attr:`allowed_formats` tuple
 
     :raise: :exc:``InvalidDateFormat <hl7apy.exceptions.InvalidDateFormat>` if the ``format`` is not in
@@ -213,11 +214,11 @@ class DateTimeDataType(BaseDataType):
 
     allowed_formats = ()
 
-    def __init__(self, value=None, format=''):
-        if format not in self.allowed_formats:
-            raise InvalidDateFormat(format)
+    def __init__(self, value=None, out_format=''):
+        if out_format not in self.allowed_formats:
+            raise InvalidDateFormat(out_format)
         self.value = value
-        self.format = format
+        self.format = out_format
 
     def to_er7(self, encoding_chars=None):
         return datetime.strftime(self.value, self.format)
@@ -233,8 +234,8 @@ class DT(DateTimeDataType):
 
     allowed_formats = ('%Y', '%Y%m', '%Y%m%d')
 
-    def __init__(self, value=None, format='%Y%m%d'):
-        super(DT, self).__init__(value, format)
+    def __init__(self, value=None, out_format='%Y%m%d'):
+        super(DT, self).__init__(value, out_format)
 
 
 class TM(DateTimeDataType):
@@ -257,8 +258,8 @@ class TM(DateTimeDataType):
 
     allowed_formats = ('%H', '%H%M', '%H%M%S', '%H%M%S.%f')
 
-    def __init__(self, value=None, format='%H%M%S.%f', offset='', microsec_precision=4):
-        super(TM, self).__init__(value, format)
+    def __init__(self, value=None, out_format='%H%M%S.%f', offset='', microsec_precision=4):
+        super(TM, self).__init__(value, out_format)
 
         if not (1 <= microsec_precision <= 4):
             raise InvalidMicrosecondsPrecision()
@@ -301,8 +302,8 @@ class DTM(TM):
     allowed_formats = ('%Y', '%Y%m', '%Y%m%d', '%Y%m%d%H', '%Y%m%d%H%M',
                        '%Y%m%d%H%M%S', '%Y%m%d%H%M%S.%f')
 
-    def __init__(self, value=None, format='%Y%m%d%H%M%S.%f', offset='', microsec_precision=4):
-        super(DTM, self).__init__(value, format, offset, microsec_precision)
+    def __init__(self, value=None, out_format='%Y%m%d%H%M%S.%f', offset='', microsec_precision=4):
+        super(DTM, self).__init__(value, out_format, offset, microsec_precision)
 
 
 class ST(TextualDataType):
@@ -338,9 +339,9 @@ class ID(TextualDataType):
     """
     def __init__(self, value, highlights=None,
                  validation_level=None):
-        #max_length is None bacause it depends from the HL7 table
+        # max_length is None bacause it depends from the HL7 table
         super(ID, self).__init__(value, None, highlights, validation_level)
-        #TODO: check for tables of allowed values: are we strict or not?
+        # TODO: check for tables of allowed values: are we strict or not?
 
 
 class IS(TextualDataType):
@@ -437,4 +438,3 @@ class TN(TextualDataType):
             raise ValueError('Invalid value for TN data')
 
         super(TN, self).__init__(value, 199, None, validation_level)
-
