@@ -23,6 +23,7 @@
 HL7apy - core classes
 """
 
+from __future__ import absolute_import
 import re
 import collections
 import datetime
@@ -40,6 +41,17 @@ from hl7apy.exceptions import ChildNotFound, ChildNotValid, \
 from hl7apy.factories import datatype_factory
 from hl7apy.base_datatypes import BaseDataType
 from hl7apy.consts import MLLP_ENCODING_CHARS
+from hl7apy.utils import iteritems
+
+try:
+    basestring = basestring
+except NameError:
+    basestring = (str, bytes)
+
+try:
+    xrange = xrange
+except NameError:
+    xrange = range
 
 
 def is_base_datatype(datatype, version=None):
@@ -106,19 +118,26 @@ class ElementProxy(collections.Sequence):
     in order to support the following API:
 
     >>> m = Message("OML_O33")
-    >>> print m.msh
+    >>> print(m.msh)
     [<Segment MSH>]
-    >>> print m.msh.__class__
-    <class '__main__.ElementProxy'>
-    >>> print m.msh.msh_7.__class__
-    <class '__main__.ElementProxy'>
+    >>> print(isinstance(m.msh, ElementProxy))
+    True
+    >>> print(isinstance(m.msh.msh_7, ElementProxy))
+    True
     """
     cls_attrs = ('element_list', 'list', 'element_name')
 
-    def __init__(self, element_list, element_name, elements):
-        self.element_name = element_name
+    def __init__(self, element_list, element_name):
+        self.element_name = element_name.upper()
         self.element_list = element_list
-        self.list = list(elements)
+
+    @property
+    def list(self):
+        return self.element_list.indexes.get(self.element_name, [])
+
+    @property
+    def traversal_list(self):
+        return self.element_list.traversal_indexes.get(self.element_name, [])
 
     def __len__(self):
         return len(self.list)
@@ -133,7 +152,10 @@ class ElementProxy(collections.Sequence):
         try:
             element = self.list[0]
         except IndexError:  # first child not found, create the element
-            element = self.element_list.create_element(self.element_name, traversal_parent=True)
+            try:
+                element = self.traversal_list[0]
+            except IndexError:
+                element = self.element_list.create_element(self.element_name, traversal_parent=True)
         return getattr(element, name)
 
     def __setattr__(self, name, value):
@@ -143,7 +165,10 @@ class ElementProxy(collections.Sequence):
             try:
                 element = self.list[0]
             except IndexError:  # first child not found, create the element
-                element = self.element_list.create_element(self.element_name, traversal_parent=True)
+                try:
+                    element = self.traversal_list[0]
+                except IndexError:
+                    element = self.element_list.create_element(self.element_name, traversal_parent=True)
             if name == 'value':
                 element.set_parent_to_traversal()
             setattr(element, name, value)
@@ -156,7 +181,6 @@ class ElementProxy(collections.Sequence):
 
     def __delitem__(self, index):
         self.element_list.remove(self.list[index])
-        del self.list[index]
 
     def __delattr__(self, name):
         delattr(self.list[0], name)
@@ -177,6 +201,8 @@ class ElementList(collections.MutableSequence):
         self.element = element
         self.list = []
         self.indexes = {}
+        self.traversal_indexes = {}
+        self.proxies = {}
 
     def get_ordered_children(self):
         """
@@ -224,11 +250,18 @@ class ElementList(collections.MutableSequence):
         :param child: an instance of an :class:`Element <hl7apy.core.Element>` subclass
         """
         if self._can_add_child(child):
-            self.list.append(child)
-            try:
-                self.indexes[child.name].append(child)
-            except KeyError:
-                self.indexes[child.name] = [child]
+            if self.element == child.parent:
+                self._remove_from_traversal_index(child)
+                self.list.append(child)
+                try:
+                    self.indexes[child.name].append(child)
+                except KeyError:
+                    self.indexes[child.name] = [child]
+            elif self.element == child.traversal_parent:
+                try:
+                    self.traversal_indexes[child.name].append(child)
+                except KeyError:
+                    self.traversal_indexes[child.name] = [child]
 
     def pop(self, index=0):
         child = super(ElementList, self).pop(index)
@@ -274,7 +307,7 @@ class ElementList(collections.MutableSequence):
         elif isinstance(value, Element):  # it is already an instance of Element
             child = value
         elif isinstance(value, BaseDataType):
-            child = self.create_element(name, True, reference)
+            child = self.create_element(name, False, reference)
             child.value = value
         else:
             raise ChildNotValid(value, child_name)
@@ -299,8 +332,11 @@ class ElementList(collections.MutableSequence):
         :type child: :class:`Element <hl7apy.core.Element>`
         :param child: an instance of :class:`Element <hl7apy.core.Element>` subclass
         """
-        self._remove_from_index(child)
-        self.list.remove(child)
+        if self.element == child.traversal_parent:
+            self._remove_from_traversal_index(child)
+        else:
+            self._remove_from_index(child)
+            self.list.remove(child)
 
     def remove_by_name(self, name, index=0):
         """
@@ -334,14 +370,23 @@ class ElementList(collections.MutableSequence):
         try:
             child = self.indexes[child_name][index]
         except (KeyError, IndexError):
-            child = None
+            try:
+                child = self.traversal_indexes[child_name][index]
+            except (KeyError, IndexError):
+                child = None
         return child
 
     def replace_child(self, old_child, new_child):
-        list_index = self.list.index(old_child)
-        by_name_index = self.indexes[old_child.name].index(old_child)
-        self.remove(old_child)
-        self.insert(list_index, new_child, by_name_index)
+        # in case it was a traversal child we just remove it from traversal children and append
+        # the new child to children
+        if old_child.traversal_parent == self.element:
+            self.remove(old_child)
+            self.append(new_child)
+        else:
+            list_index = self.list.index(old_child)
+            by_name_index = self.indexes[old_child.name].index(old_child)
+            self.remove(old_child)
+            self.insert(list_index, new_child, by_name_index)
 
     def create_element(self, name, traversal_parent=False, reference=None):
         """
@@ -399,31 +444,36 @@ class ElementList(collections.MutableSequence):
 
         :return: an instance of :class:`ElementProxy <hl7apy.core.ElementProxy>` containing the results
         """
-        if name in self.indexes:  # use the indexes to find the children faster
-            results = ElementProxy(self, name, (c for c in self.indexes[name]))
-            return results
+        if name in self.indexes or name in self.traversal_indexes:
+            try:
+                return self.proxies[name]
+            except KeyError:
+                self.proxies[name] = ElementProxy(self, name)
+                return self.proxies[name]
         else:  # child not found in the indexes dictionary (e.g. msh_9.message_code, msh_9.msh_9_1)
             child_name = self._find_name(name)
             if child_name is not None:
-                results = ElementProxy(self, name, (c for c in self.indexes.get(child_name, [])))
-                return results
+                try:
+                    return self.proxies[child_name]
+                except KeyError:
+                    self.proxies[child_name] = ElementProxy(self, child_name)
+                    return self.proxies[child_name]
 
     def _can_add_child(self, child):
         if self.element._is_valid_child(child):
-            if child.parent != self.element:  # avoid infinite recursion
+            if child.parent != self.element and child.traversal_parent != self.element:  # avoid infinite recursion
                 child.parent = self.element
             else:
                 # if validation is strict, check the child cardinality
                 if Validator.is_strict(self.element.validation_level):
-                    min, max = self.element.repetitions.get(child.name, (0, -1))
-                    if len(self.indexes.get(child.name, [])) + 1 > int(max) and max > -1:
-                        raise MaxChildLimitReached(self.element, child, max)
+                    min_rep, max_rep = self.element.repetitions.get(child.name, (0, -1))
+                    if len(self.indexes.get(child.name, [])) + 1 > int(max_rep) and max_rep > -1:
+                        raise MaxChildLimitReached(self.element, child, max_rep)
                 if self.element.validation_level != child.validation_level:
                     raise OperationNotAllowed('Cannot add a child with a different validation_level')
                 if self.element.version != child.version:
                     raise OperationNotAllowed('Cannot add a child with a different HL7 version')
                 return True
-
         else:
             raise ChildNotValid(child, self.element)
         return False
@@ -434,6 +484,14 @@ class ElementList(collections.MutableSequence):
         except (KeyError, ValueError):
             pass
 
+    def _remove_from_traversal_index(self, child):
+        try:
+            self.traversal_indexes[child.name].remove(child)
+            if len(self.traversal_indexes[child.name]) == 0:
+                del self.traversal_indexes[child.name]
+        except (KeyError, ValueError):
+            pass
+
     def __len__(self):
         return len(self.list)
 
@@ -554,9 +612,9 @@ class Element(object):
     """
 
     cls_attrs = ['name', 'validation_level', 'version', 'children', 'ordered_children',
-                 'table', 'long_name', 'value', '_value', 'parent',  '_parent', 'traversal_parent',
-                 'child_classes', 'encoding_chars', 'structure_by_name', 'structure_by_longname',
-                 'repetitions', 'reference']
+                 'table', 'long_name', 'value', '_value', 'parent',  '_parent', '_traversal_parent',
+                 'traversal_parent', 'child_classes', 'encoding_chars', 'structure_by_name',
+                 'structure_by_longname', 'repetitions', 'reference']
 
     def __init__(self, name=None, parent=None, reference=None, version=None,
                  validation_level=None, traversal_parent=None):
@@ -613,7 +671,7 @@ class Element(object):
         >>> f = Field('PID_5')
         >>> f.value = 'EVERYMAN^ADAM'
         >>> s.add(f)
-        >>> print s.to_er7()
+        >>> print(s.to_er7())
         PID|||||EVERYMAN^ADAM
         """
         self.children.append(obj)
@@ -701,13 +759,23 @@ class Element(object):
 
     def _set_parent(self, parent):
         self._parent = parent
-        self.traversal_parent = None
         if parent is not None:
+            self.traversal_parent = None
             self.parent.add(self)
 
     parent = property(_get_parent, _set_parent,
                       doc="The parent :class:`Element <hl7apy.core.Element>` of this one")
 
+    def _get_traversal_parent(self):
+        return self._traversal_parent
+
+    def _set_traversal_parent(self, parent):
+        self._traversal_parent = parent
+        if parent is not None:
+            parent.add(self)
+
+    traversal_parent = property(_get_traversal_parent, _set_traversal_parent)
+
     def _set_value(self, value):
         self.children = self.parse_children(value)
 
@@ -750,7 +818,7 @@ class Element(object):
     def _find_structure(self, reference=None):
         if self.name is not None:
             structure = ElementFinder.get_structure(self, reference)
-            for k, v in structure.iteritems():
+            for k, v in iteritems(structure):
                 setattr(self, k, v)
 
     def _is_valid_child(self, child):
@@ -847,7 +915,7 @@ class SupportComplexDataType(Element):
                 datatype not in ('varies', None, self.datatype):
             reference = load_reference(datatype, 'Component', self.version)
             structure = ElementFinder.get_structure(self, reference)
-            for k, v in structure.iteritems():
+            for k, v in iteritems(structure):
                 setattr(self, k, v)
 
         if hasattr(self, 'children') and len(self.children) >= 1:
@@ -1040,7 +1108,7 @@ class SubComponent(CanBeVaries):
 
         >>> s = SubComponent("CE_1")
         >>> s.value = "IDENTIFIER"
-        >>> print s.to_er7()
+        >>> print(s.to_er7())
         IDENTIFIER
         """
 
@@ -1142,7 +1210,8 @@ class Component(SupportComplexDataType, CanBeVaries):
             raise OperationNotAllowed("Cannot instantiate an unknown Element with strict validation")
 
         # TODO: This control should be deleted (see CanBeVaries)
-        if datatype is not None and Validator.is_strict(validation_level) and self.datatype != 'varies' and self.datatype != datatype:
+        if datatype is not None and Validator.is_strict(validation_level) and \
+                self.datatype != 'varies' and self.datatype != datatype:
             raise OperationNotAllowed("Cannot assign a different datatype with strict validation")
 
     def add_subcomponent(self, name):
@@ -1154,9 +1223,9 @@ class Component(SupportComplexDataType, CanBeVaries):
 
         >>> c = Component(datatype='CE')
         >>> ce_1 = c.add_subcomponent('CE_1')
-        >>> print ce_1
+        >>> print(ce_1)
         <SubComponent CE_1>
-        >>> print ce_1 in c.children
+        >>> print(ce_1 in c.children)
         True
         """
         if self.is_unknown() and is_base_datatype(self.datatype):
@@ -1175,7 +1244,7 @@ class Component(SupportComplexDataType, CanBeVaries):
         >>> s2 = SubComponent(name='CWE_4', value='ALT_ID')
         >>> c.add(s)
         >>> c.add(s2)
-        >>> print c.to_er7()
+        >>> print(c.to_er7())
         EXAMPLE_ID&&&ALT_ID
         """
         # base datatype components can't have more than one child
@@ -1276,7 +1345,7 @@ class Field(SupportComplexDataType):
         :return: an instance of :class:`Component <hl7apy.core.Component>`
 
         >>> s = Field('PID_5')
-        >>> print s.add_component('XPN_2')
+        >>> print(s.add_component('XPN_2'))
         <Component XPN_2 (GIVEN_NAME) of type ST>
         """
         return self.children.create_element(name)
@@ -1310,7 +1379,7 @@ class Field(SupportComplexDataType):
         >>> c = Component('XPN_2')
         >>> c.value = 'ADAM'
         >>> f.add(c)
-        >>> print f.to_er7()
+        >>> print(f.to_er7())
         EVERYMAN^ADAM
         """
         # base datatype components can't have more than one child
@@ -1345,7 +1414,7 @@ class Field(SupportComplexDataType):
 
         >>> msh_9 = Field("MSH_9")
         >>> msh_9.value = "ADT^A01^ADT_A01"
-        >>> print msh_9.to_er7()
+        >>> print(msh_9.to_er7())
         ADT^A01^ADT_A01
         """
         if encoding_chars is None:
@@ -1528,7 +1597,7 @@ class Segment(Element):
         :return: an instance of :class:`Field <hl7apy.core.Field>`
 
         >>> s = Segment('PID')
-        >>> print s.add_field('PID_1')
+        >>> print(s.add_field('PID_1'))
         <Field PID_1 (SET_ID_PID) of type SI>
         """
         return self.children.create_element(name)
@@ -1598,7 +1667,7 @@ class Segment(Element):
         >>> pid = Segment("PID")
         >>> pid.pid_1 = '1'
         >>> pid.pid_5 = "EVERYMAN^ADAM"
-        >>> print pid.to_er7()
+        >>> print(pid.to_er7())
         PID|1||||EVERYMAN^ADAM
         """
         if encoding_chars is None:
@@ -1693,9 +1762,9 @@ class Group(Element):
 
         >>> m = Message('QBP_Q11')
         >>> qpd = m.add_segment('QPD')
-        >>> print qpd
+        >>> print(qpd)
         <Segment QPD>
-        >>> print qpd in m.children
+        >>> print(qpd in m.children)
         True
         """
         return self.children.create_element(name)
@@ -1709,9 +1778,9 @@ class Group(Element):
 
         >>> m = Message('OML_O33')
         >>> patient = m.add_group('OML_O33_PATIENT')
-        >>> print patient
+        >>> print(patient)
         <Group OML_O33_PATIENT>
-        >>> print patient in m.children
+        >>> print(patient in m.children)
         True
         """
 
