@@ -19,6 +19,7 @@
 # IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import absolute_import
 import re
 
 from hl7apy import get_default_encoding_chars, get_default_version, \
@@ -28,8 +29,14 @@ from hl7apy.core import is_base_datatype, Message, Group, Segment, Field, Compon
 from hl7apy.exceptions import InvalidName, ParserError, InvalidEncodingChars, MessageProfileNotFound
 from hl7apy.validation import Validator
 
+try:
+    xrange = xrange
+except NameError:
+    xrange = range
 
-def parse_message(message, validation_level=None, find_groups=True, message_profile=None, report_file=None):
+
+def parse_message(message, validation_level=None, find_groups=True, message_profile=None, report_file=None,
+                  force_validation=False):
     """
     Parse the given ER7-encoded message and return an instance of :class:`Message <hl7apy.core.Message>`.
 
@@ -45,16 +52,20 @@ def parse_message(message, validation_level=None, find_groups=True, message_prof
     :param find_groups: if ``True``, automatically assign the segments found to the appropriate
         :class:`Groups <hl7apy.core.Group>` instances. If ``False``, the segments found are assigned as
         children of the :class:`Message <hl7apy.core.Message>` instance
+        
+    :type force_validation: ``bool``
+    :type force_validation: if ``True``, automatically forces the message validation after the end of the parsing
 
     :return: an instance of :class:`Message <hl7apy.core.Message>`
 
-    >>> message = "MSH|^~\&|GHH_ADT||||20080115153000||OML^O33^OML_O33|0123456789|P|2.5||||AL\\rPID|1||566-554-3423^^^GHH^MR||EVERYMAN^ADAM^A|||M|||2222 HOME STREET^^ANN ARBOR^MI^^USA||555-555-2004|||M\\r"
+    >>> message = "MSH|^~\&|GHH_ADT||||20080115153000||OML^O33^OML_O33|0123456789|P|2.5||||AL\\rPID|1||" \
+    "566-554-3423^^^GHH^MR||EVERYMAN^ADAM^A|||M|||2222 HOME STREET^^ANN ARBOR^MI^^USA||555-555-2004|||M\\r"
     >>> m = parse_message(message)
-    >>> print m
+    >>> print(m)
     <Message OML_O33>
-    >>> print m.msh.sending_application.to_er7()
+    >>> print(m.msh.sending_application.to_er7())
     GHH_ADT
-    >>> print m.children
+    >>> print(m.children)
     [<Segment MSH>, <Group OML_O33_PATIENT>]
     """
     message = message.lstrip()
@@ -80,7 +91,8 @@ def parse_message(message, validation_level=None, find_groups=True, message_prof
         create_groups(m, children, validation_level)
     else:
         m.children = children
-    if Validator.is_strict(validation_level):
+
+    if force_validation:
         if message_profile is None:
             Validator.validate(m, report_file=report_file)
         else:
@@ -114,8 +126,9 @@ def parse_segments(text, version=None, encoding_chars=None, validation_level=Non
 
     :return: a list of :class:`Segment <hl7apy.core.Segment>` instances
 
-    >>> segments = "EVN||20080115153000||||20080114003000\\rPID|1||566-554-3423^^^GHH^MR||EVERYMAN^ADAM^A|||M|||2222 HOME STREET^^ANN ARBOR^MI^^USA||555-555-2004|||M\\r"
-    >>> print parse_segments(segments)
+    >>> segments = "EVN||20080115153000||||20080114003000\\rPID|1||566-554-3423^^^GHH^MR||EVERYMAN^ADAM^A|||M|||" \
+    "2222 HOME STREET^^ANN ARBOR^MI^^USA||555-555-2004|||M\\r"
+    >>> print(parse_segments(segments))
     [<Segment EVN>, <Segment PID>]
     """
     version = _get_version(version)
@@ -167,9 +180,9 @@ def parse_segment(text, version=None, encoding_chars=None, validation_level=None
 
     >>> segment = "EVN||20080115153000||||20080114003000"
     >>> s =  parse_segment(segment)
-    >>> print s
+    >>> print(s)
     <Segment EVN>
-    >>> print s.to_er7()
+    >>> print(s.to_er7())
     EVN||20080115153000||||20080114003000
     """
     version = _get_version(version)
@@ -220,15 +233,16 @@ def parse_fields(text, name_prefix=None, version=None, encoding_chars=None, vali
 
     >>> fields = "1|NUCLEAR^NELDA^W|SPO|2222 HOME STREET^^ANN ARBOR^MI^^USA"
     >>> nk1_fields = parse_fields(fields, name_prefix="NK1")
-    >>> print nk1_fields
-    [<Field NK1_1 (SET_ID_NK1) of type SI>, <Field NK1_2 (NAME) of type XPN>, <Field NK1_3 (RELATIONSHIP) of type CE>, <Field NK1_4 (ADDRESS) of type XAD>]
+    >>> print(nk1_fields)
+    [<Field NK1_1 (SET_ID_NK1) of type SI>, <Field NK1_2 (NAME) of type XPN>, <Field NK1_3 (RELATIONSHIP) of type CE>, \
+<Field NK1_4 (ADDRESS) of type XAD>]
     >>> s = Segment("NK1")
     >>> s.children = nk1_fields
-    >>> print s.to_er7()
+    >>> print(s.to_er7())
     NK1|1|NUCLEAR^NELDA^W|SPO|2222 HOME STREET^^ANN ARBOR^MI^^USA
     >>> unknown_fields = parse_fields(fields)
     >>> s.children = unknown_fields
-    >>> print s.to_er7()
+    >>> print(s.to_er7())
     NK1||||||||||||||||||||||||||||||||||||||||1|NUCLEAR^NELDA^W|SPO|2222 HOME STREET^^ANN ARBOR^MI^^USA
     """
     version = _get_version(version)
@@ -298,14 +312,14 @@ def parse_field(text, name=None, version=None, encoding_chars=None, validation_l
 
     >>> field = "NUCLEAR^NELDA^W"
     >>> nk1_2 = parse_field(field, name="NK1_2")
-    >>> print nk1_2
+    >>> print(nk1_2)
     <Field NK1_2 (NAME) of type XPN>
-    >>> print nk1_2.to_er7()
+    >>> print(nk1_2.to_er7())
     NUCLEAR^NELDA^W
     >>> unknown = parse_field(field)
-    >>> print unknown
+    >>> print(unknown)
     <Field of type None>
-    >>> print unknown.to_er7()
+    >>> print(unknown.to_er7())
     NUCLEAR^NELDA^W
     """
     version = _get_version(version)
@@ -368,10 +382,13 @@ def parse_components(text, field_datatype='ST', version=None, encoding_chars=Non
 
     >>> components = "NUCLEAR^NELDA^W^^TEST"
     >>> xpn = parse_components(components, field_datatype="XPN")
-    >>> print xpn
-    [<Component XPN_1 (FAMILY_NAME) of type FN>, <Component XPN_2 (GIVEN_NAME) of type ST>, <Component XPN_3 (SECOND_AND_FURTHER_GIVEN_NAMES_OR_INITIALS_THEREOF) of type ST>, <Component XPN_5 (PREFIX_E_G_DR) of type ST>]
-    >>> print parse_components(components)
-    [<Component ST (None) of type ST>, <Component ST (None) of type ST>, <Component ST (None) of type ST>, <Component ST (None) of type ST>, <Component ST (None) of type ST>]
+    >>> print(xpn)
+    [<Component XPN_1 (FAMILY_NAME) of type FN>, <Component XPN_2 (GIVEN_NAME) of type ST>, \
+<Component XPN_3 (SECOND_AND_FURTHER_GIVEN_NAMES_OR_INITIALS_THEREOF) of type ST>, \
+<Component XPN_5 (PREFIX_E_G_DR) of type ST>]
+    >>> print(parse_components(components))
+    [<Component ST (None) of type ST>, <Component ST (None) of type ST>, <Component ST (None) of type ST>, \
+<Component ST (None) of type ST>, <Component ST (None) of type ST>]
     """
     version = _get_version(version)
     encoding_chars = _get_encoding_chars(encoding_chars)
@@ -439,11 +456,11 @@ def parse_component(text, name=None, datatype='ST', version=None, encoding_chars
 
     >>> component = "GATEWAY&1.3.6.1.4.1.21367.2011.2.5.17"
     >>> cx_4 = parse_component(component, name="CX_4")
-    >>> print cx_4
+    >>> print(cx_4)
     <Component CX_4 (ASSIGNING_AUTHORITY) of type None>
-    >>> print cx_4.to_er7()
+    >>> print(cx_4.to_er7())
     GATEWAY&1.3.6.1.4.1.21367.2011.2.5.17
-    >>> print parse_component(component)
+    >>> print(parse_component(component))
     <Component ST (None) of type None>
     """
     version = _get_version(version)
@@ -495,17 +512,17 @@ def parse_subcomponents(text, component_datatype='ST', version=None, encoding_ch
 
     >>> subcomponents= "ID&TEST&&AHAH"
     >>> cwe = parse_subcomponents(subcomponents, component_datatype="CWE")
-    >>> print cwe
+    >>> print(cwe)
     [<SubComponent CWE_1>, <SubComponent CWE_2>, <SubComponent CWE_4>]
     >>> c = Component(datatype='CWE')
     >>> c.children = cwe
-    >>> print c.to_er7()
+    >>> print(c.to_er7())
     ID&TEST&&AHAH
     >>> subs = parse_subcomponents(subcomponents)
-    >>> print subs
+    >>> print(subs)
     [<SubComponent ST>, <SubComponent ST>, <SubComponent ST>, <SubComponent ST>]
     >>> c.children = subs
-    >>> print c.to_er7()
+    >>> print(c.to_er7())
     &&&&&&&&&ID&TEST&&AHAH
     """
     version = _get_version(version)
@@ -559,16 +576,8 @@ def parse_subcomponent(text, name=None, datatype='ST', version=None, validation_
                         validation_level=validation_level)
 
 
-def get_message_info(content):
-    """
-    Parse the given ER7-encoded message to find its version, structure and encoding chars
-
-    :param content: the ER7-encoded message
-
-    :return: a tuple containing (encoding_chars, message_structure, version)
-    """
-    regex = re.compile("^MSH(?P<field_sep>\S)")
-    m = regex.match(content)
+def _split_msh(content):
+    m = re.match("^MSH(?P<field_sep>\S)", content)
     if m is not None:  # if the regular expression matches, it is an HL7 message
         field_sep = m.group('field_sep')  # get the field separator (first char after MSH)
         msh = content.split("\r", 1)[0]  # get the first segment
@@ -596,6 +605,23 @@ def get_message_info(content):
     else:
         raise ParserError("Invalid message")
 
+    return fields, encoding_chars
+
+
+def get_message_type(content):
+    fields, enc_chars = _split_msh(content)
+
+    try:
+        msh_9 = fields[8].strip()
+    except IndexError:
+        msh_9 = None
+
+    return msh_9
+
+
+def get_message_info(content):
+    fields, encoding_chars = _split_msh(content)
+
     # look for MSH.9 field (e.g. ADT^A01^ADT_A01) containing the message structure
     try:
         msh_9 = fields[8].strip()
@@ -693,7 +719,8 @@ def _find_group(segment, search_data, validation_level=None):
                   if structure['structure_by_name'][k]['cls'] == Group]
         # for any group found, create the group and check if the segment is one of its children
         for g in groups:
-            group = Group(g, version=segment.version, validation_level=validation_level)
+            group = Group(g, version=segment.version, reference=structure['structure_by_name'][g]['ref'],
+                          validation_level=validation_level)
             p_structure = {'structure_by_name':  group.structure_by_name,
                            'ordered_children': group.ordered_children,
                            'repetitions': group.repetitions}
